/*  ------------------------------------------------------- !
! The current OpenMP code is to:                            !
! (1) count to the number of even integer in a matrix.      !
! (2) demonstrate the false sharing issues.                 !
!-----------------------------------------------------------!
! this is the OpenMP version v1.                            !
! (1) coarse-grained partition along the row direction (i): !
!     each thread covers the rows from start_idx to end_idx,!
!     and all columns.                                      !
! (2) the shared counter[] array stores the contribution    !
!     from each thread.                                     !
!-----------------------------------------------------------!
! Xiaoxu Guan at LSU HPC.                                   !
! March 22, 2017                                            !
!--------------------------------------------------------- */

# include <omp.h>
# include <stdio.h>
# include <stdlib.h>
# include <math.h>
#define MIN(a,b) (((a) < (b)) ? (a) : (b))

int main() {

int i,j,nosize,counter_even,itmp;
//const int nsize = 100000;
const int nsize = 10000;
double tmp,time_start,time_end;
int **matrix;
int *counter,*start_idx,*end_idx;
int id,nothread,chunk_size;

  nosize = nsize;

// allocate the matrix.
  matrix = (int **) malloc( nosize * sizeof(int *) );
  if(matrix == NULL) printf("Failure to allocate array matrix!\n");

  for (i=0; i<nosize; i++) 
      { matrix[i] = (int *) malloc( nosize*sizeof(int) );
        if(matrix[i] == NULL) { printf("Failure to allocate array element matrix[i] %d\n",i); exit(0); }
      }

// initialize the matrix.
   for (i=0; i<nosize; i++)
   for (j=0; j<nosize; j++)
     {  tmp = (double) (i*j+1) / (double) (i+j+1);
        matrix[i][j] = (int) (tmp);
     }

#pragma omp parallel
{
   id = omp_get_thread_num();
   nothread = omp_get_num_threads();
   printf("This is ID %d out of %d\n",id,nothread);
}
 


// start a timer.
   time_start = omp_get_wtime();


// define arrays.
   counter = (int *) malloc( nothread*sizeof(int) );
   start_idx = (int *) malloc( nothread*sizeof(int) );
   end_idx = (int *) malloc( nothread*sizeof(int) );
   chunk_size = nosize / nothread + 1;
   for (id=0; id<nothread; id++) 
       { start_idx[id] = id*chunk_size;
         end_idx[id] = MIN(start_idx[id] + chunk_size-1, nosize-1);
         printf("my id is %d, start_idx = %d, end_idx = %d\n", id,start_idx[id],end_idx[id]);
       } 



// count the number of even integers.
   for(id = 0; id < nothread; id++)  counter[id] = 0;

#pragma omp parallel private(id,i,j,itmp) 
 {  id = omp_get_thread_num();
    for (i = start_idx[id]; i <= end_idx[id]; i++)
    for (j = 0; j < nosize; j++) 
     {  itmp = matrix[i][j]%2;
        if (itmp == 0) ++counter[id];
     }
 }

   counter_even = 0;
   for(id = 0; id < nothread; id++)  counter_even += counter[id];

   time_end = omp_get_wtime();
   printf("Number of even integers is %d\n", counter_even);
   printf("Elapsed time (sec) is %.5f\n", time_end - time_start);

   free(matrix);
   free(counter);
   free(start_idx);
   free(end_idx);

}

