/*  ----------------------------------------------------- !
! The current OpenMP code is to:                          !
! (1) demonstrate the false sharing issues.               !
! (2) use 'parallel for' wihout manual partition.         !
!---------------------------------------------------------!
! this is the OpenMP version v3.                          !
!---------------------------------------------------------!
! (1) the shared counter array was removed.               !
! (2) use the 'critical' pragma to control the access to  !
!     the shared scalar counter_even.                     !
!---------------------------------------------------------!
! Xiaoxu Guan at LSU HPC.                                 !
! March 22, 2017                                          !
!------------------------------------------------------- */

# include <omp.h>
# include <stdio.h>
# include <stdlib.h>
# include <math.h>


int main() {

int i,j,nosize,counter_even,itmp;
const int nsize = 10000;
double tmp,time_start,time_end;
int  **matrix;

  nosize = nsize;

// allocate the matrix.
  matrix = (int **) malloc( nosize * sizeof(int *) );
  if(matrix == NULL) printf("Failure to allocate array matrix!\n");

  for (i=0; i<nosize;i++) 
      { matrix[i] = (int *) malloc(nosize*sizeof(int));
        if(matrix[i] == NULL) { printf("Failure to allocate array element matrix[i] %d\n",i); exit(0); }
      }

// initialize the matrix.
   for (i=0; i<nosize; i++)
   for (j=0; j<nosize; j++)
     {  tmp = (double) (i*j+1) / (double) (i+j+1);
        matrix[i][j] = (int) (tmp);
     }

// start a timer.
   time_start = omp_get_wtime();

// count the number of even integers.
   counter_even = 0;
#pragma omp parallel for private(i,j,itmp)
   for (i=0; i<nosize; i++)
   for (j=0; j<nosize; j++)
     {  itmp = matrix[i][j]%2;
#pragma omp critical
        if (itmp == 0) ++counter_even;
     }


   time_end = omp_get_wtime();
   printf("Number of even integers is %d\n", counter_even);
   printf("Elapsed time (sec) is %.5f\n", time_end - time_start);

}

