/*  ------------------------------------------------------------ !
! The current OpenMP code is to:                                 !
! (1) see how to jump out of the inner loop.                     !
! (2) see what happens if we want to jump out of the outer loop. !
! (3) see how a compiler responses during compilation.           !
! (4) assume the outer loop is parallelized.                     !
!----------------------------------------------------------------!
! Xiaoxu Guan at LSU HPC.                                        !
! May 24, 2017                                                   !
!-------------------------------------------------------------- */

#include <omp.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <stdbool.h>

int main () {

   int i,j,id,nothread;
   const int nsize = 100;
   double tsum,time_start,time_end;
   double **a,**b;
   int  nosize;
   bool irun;
  
   nosize = nsize;
   
   a = malloc( nsize * sizeof(double *) );
   b = malloc( nsize * sizeof(double *) );
   if(a == NULL) printf("Failure to allocate array a!\n");
   if(b == NULL) printf("Failure to allocate array b!\n");

   for(i = 0; i < nosize; i++) 
   {
      a[i] = malloc( nosize * sizeof(double) ); 
      if(a[i] == NULL) { printf("Failure to allocate array element a[i] %d\n",i); exit(0); }

      b[i] = malloc( nosize * sizeof(double) ); 
      if(b[i] == NULL) { printf("Failure to allocate array element b[i] %d\n",i); exit(0); }
   }


/* initialize arrays: a and b */
   for(i = 0; i < nosize; i++)  {
   for(j = 0; j < nosize; j++)  {
       a[i][j] = sin( (double) (i-j) );
       b[i][j] = cos( (double) (i+j) );
     } }


  time_start = omp_get_wtime();

   irun = true;
#pragma omp parallel for default(none) private(i,j,tsum) firstprivate(irun) shared(nosize,a,b)
   for(i = 0; i < nosize; i++) {
   for(j = 0; j < nosize; j++) { 
          tsum = a[i][j]+ b[i][j];
       if(tsum > 0.5) {
          printf("My ID is %d i = %d j = %d tsum = %9.6f\n",omp_get_thread_num(),i,j,tsum);
          irun = false ;
          break;
       }
   }
//       if(!irun) { break; }
   }

  time_end = omp_get_wtime();
  printf("elapsed time (second) = %.5f\n", time_end - time_start);
  free(a);
  free(b);

}
