/*  --------------------------------------------------- !
! The current OpenMP code is to:                        !
! (1) see how to parallalize nested loops.              !
! (2) use a naive version of matrix-matrix product.     !
!-------------------------------------------------------!
! Xiaoxu Guan at LSU HPC.                               !
! March 20, 2017                                        !
!----------------------------------------------------- */

#include <omp.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>


int main () {

   int i,j,k,id,nothread,nosize,index,inda,indb;
   const int nsize = 4000, nsize2 = nsize*nsize;
   double temp,time_start,time_end;
   double a[nsize2],b[nsize2],c[nsize2];


   nosize = nsize;
   index = -1;
/* initialize arrays: a, b, and c:   */
/* coulumn: j and row: i. */
   for(i = 0; i < nosize; i++)  {
   for(j = 0; j < nosize; j++)  {
       index += 1;
       a[index] = sin( (double) (i-j) );
       b[index] = cos( (double) (i+j) );
       c[index] = 0.0;
     } }


time_start = omp_get_wtime();

/* only parallelize the outer loop with ijk: */
#pragma omp parallel for default(none) private(i,j,k,temp,index,inda,indb) shared(nosize,a,b,c)
   for(i = 0; i < nosize; i++)  { 
   for(j = 0; j < nosize; j++)  { 
           index = i*nosize + j;
           temp = 0.0; 
       for(k = 0; k < nosize; k++)  { 
           inda =  i*nosize + k;
           indb =  k*nosize + j;
           temp += a[inda] * b[indb]; 
           }
       c[index] = temp;
   } } 


time_end = omp_get_wtime();

printf("c[0] = %15.5f\n",c[0]);
printf("elapsed time (second) = %.5f\n", time_end - time_start);



time_start = omp_get_wtime();

/* parallelize two outer loops with the same algorithm (ijk). */
#pragma omp parallel for default(none) private(i,j,k,temp,index,inda,indb) shared(nosize,a,b,c) \
        collapse(2) 
   for(i = 0; i < nosize; i++)  {
   for(j = 0; j < nosize; j++)  {
          index = i*nosize + j;
          temp = 0.0; 
       for(k = 0; k < nosize; k++)  {
          inda =  i*nosize + k;
          indb =  k*nosize + j;
          temp += a[inda] * b[indb];
          }
       c[index] = temp;
   } }


time_end = omp_get_wtime();

printf("c[0] = %15.5f\n",c[0]);
printf("elapsed time (second) = %.5f\n", time_end - time_start);


}
