/*  --------------------------------------------------- !
! The current OpenMP code is to:                        !
! (1) do matrix-matrix products.                        !
! (2) see the behavior of the threaded MKL routine.     !
! (3) thread control at different levels.               !
! (4) precision: real DP.                               !
!-------------------------------------------------------!
! Xiaoxu Guan at LSU HPC.                               !
! March 30, 2017                                        !
!----------------------------------------------------- */

#include <omp.h>
#include <mkl.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>

int main() {
	
const int nsize = 5000;
const int nsize2 = nsize*nsize;
const int iteration = 1600;
const double invs_nsize = 1.0/(double)(nsize);
const double alpha = 1.0, beta = 0.0;
double *matrix_a, *matrix_b, *matrix_c;
char trans_a = 'N', trans_b = 'N';
int i,j,k;

// allocate the matrices.
    matrix_a = (double *) malloc( nsize2*sizeof(double) );     
    matrix_b = (double *) malloc( nsize2*sizeof(double) );     
    matrix_c = (double *) malloc( nsize2*sizeof(double) );     
    if(matrix_a == NULL || matrix_b == NULL || matrix_c == NULL) printf("Failure to allocate array matrix!\n");

// initialize the matrices.
    for (i=0; i<nsize2; i++)
	{ matrix_a[i] = sin((double)(i))*invs_nsize; 
          matrix_b[i] = cos((double)(i))*invs_nsize; 
          matrix_c[i] = 0.0; 
        }



// repeat "iteration" times: C = A*B.
// #pragma omp parallel for num_threads(4)
 #pragma omp parallel for 
    for (k=0; k<iteration; k++)
      {  
        printf("Number of the threads = %d\n", omp_get_num_threads());
        cblas_dgemm(CblasRowMajor, CblasNoTrans, CblasNoTrans, nsize, nsize, nsize, \
                    alpha, matrix_a, nsize, matrix_b, nsize, beta, matrix_c, nsize);
      }

// print the a corner in the 1st row of the C matrix.
    for (i=0; i<=5; i++)
         printf("%.5e\n", matrix_c[i]); 

    printf("\n");
    free (matrix_a);
    free (matrix_b);
    free (matrix_c);

}
