/* --------------------------------------------------------------!
! The current OpenMP code is to:                                 !
! (1) use Intel and GCC compilers and see how                    !
!     OMP_NUM_THREADS was set by default.                        !  
! (2) use threadprivate variable in the different program units. ! 
!----------------------------------------------------------------!
! Xiaoxu Guan at LSU HPC.                                        !
! April 14, 2017                                                 !
!-------------------------------------------------------------- */

#include <omp.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#define nsize 410


/* global variables */
int id, istart,iend;
double array_a[nsize],array_b[nsize],array_c[nsize];
#pragma omp threadprivate (id,istart,iend) 
/* end of global variables */


int main () {

int i,nothread,nchunk;
double diff;
double array_master[nsize-1];


   for(i = 0; i < nsize; i++ ) 
   {
      array_a[i] = (double) i;
      array_b[i] = (double) (i+i);
   }


#pragma omp parallel
    {
      nothread = omp_get_num_threads();
      id = omp_get_thread_num();
      nchunk = (double)(nsize) / (double)(nothread);
      istart = id*nchunk;
      iend = istart + nchunk - 1;
      if(id == nothread-1) {
         istart = id*nchunk;
         iend = nsize-1;
         nchunk = iend - istart + 1;
        }
       printf("Hello from %5d %5d %5d\n", id, istart, iend);
    }

       printf("\n");
       printf("Hello from master. id = %d\n\n", id);


#pragma omp parallel 
    {
      scalar();
    }

/* check the results:  */
      for( i = 1; i < nsize; i++) {
         array_master[i] = array_a[i] + array_b[i];
         diff = ( array_master[i] - array_c[i] ) / array_master[i];
         if( fabs(diff) >= 1.e-10 ) {
             printf("Error seems too large!\n");
             printf("i = %d diff = %f %f %f\n", i, diff, array_master[i], array_c[i]);
            } 
      }

}   /* end of main */




/*--------------------------------------------------!
! the function needs to be called by all threads.   !
!--------------------------------------------------*/
int scalar() {
int i;

/* each thread only works on its own chunk of arrays.  */
   printf("from the scalar function. id = %5d  istart = %5d  iend = %5d\n", id, istart, iend);

   for (i = istart; i <= iend; i++) {
        array_c[i] = array_a[i] + array_b[i];
       }
return 0;
}


