!----------------------------------------------------------------!
! The current OpenMP code is to:                                 !
! (1) count to the number of even integer in a matrix.           !
! (2) demonstrate the false sharing issues.                      !
!----------------------------------------------------------------!
! this is the OpenMP version v1.                                 !
! (1) coarse-grained partition along the row direction (i):      !
!     each thread covers the rows from start_idx to end_idx,     !
!     and all columns.                                           !
! (2) the shared counter[] array stores the contribution         !
!     from each thread.                                          !
!----------------------------------------------------------------!
! Xiaoxu Guan at LSU HPC.                                        !
! March 17, 2017                                                 !
!----------------------------------------------------------------!

      program false_sharing
      implicit none
      integer                                 :: i,j,nosize,counter_even,itmp
      integer                                 :: id,nothread,omp_get_thread_num,  &
                                                 omp_get_num_threads,chunk_size
!!      integer, parameter                      :: nsize = 100000
      integer, parameter                      :: nsize = 10000
      real(kind=8)                            :: tmp,time_start,time_end,omp_get_wtime
      integer, dimension(0:nsize-1,0:nsize-1) :: matrix
      integer, dimension(:), allocatable      :: counter,start_idx,end_idx

! initialize the matrix.
      nosize = nsize
      do i = 0, nosize-1
      do j = 0, nosize-1
         tmp = dble(i*j+1) / dble(i+j+1)
         matrix(j,i) = int(tmp)
      end do
      end do


!$omp parallel private (id,i) shared(nothread)
      id = omp_get_thread_num()
      nothread = omp_get_num_threads()
      write(*,'(1x,a,i2,a,i2)') &
     &   "This is ID ", id, " out of ", nothread
!$omp end parallel


      time_start = omp_get_wtime()
 
      ALLOCATE( counter(0:nothread-1) )
      ALLOCATE( start_idx(0:nothread-1) )
      ALLOCATE( end_idx(0:nothread-1) )

      chunk_size = nosize / nothread + 1
      do id = 0, nothread-1
         start_idx(id) = id * chunk_size 
         end_idx(id) = min0(start_idx(id) + chunk_size - 1, nosize-1)
         write(*,'(a,i3,a,i6,a,i6)') ' my id is ',id,  &
                   ',   start_idx = ',start_idx(id), ',  end_idx = ',end_idx(id)
      end do


! count the number of even integers.
      counter = 0
!$omp parallel private(id,i,j,itmp) shared(start_idx,end_idx,nosize,counter,matrix)
         id = omp_get_thread_num()
         do i = start_idx(id), end_idx(id)
         do j = 0, nosize-1
            itmp = mod(matrix(j,i),2)
            if (itmp == 0) counter(id) = counter(id) + 1
         end do
         end do
!$omp end parallel

      time_end = omp_get_wtime()
      write(*,'(1x,a,i12)') "Number of even integers is ", sum(counter)
      write(*,'(1x,a,f10.5)') "Elapsed time (sec) is ", time_end - time_start

      DEALLOCATE(counter,start_idx,end_idx)

      end program false_sharing

