!----------------------------------------------------------------!
! The current OpenMP code is to:                                 !
! (1) see how to jump out of the inner loop.                     !
! (2) see what happens if we want to jump out of the outer loop. !
! (3) see how a compiler responses during compilation.           !
! (4) assume the outer loop is parallelized.                     !
!----------------------------------------------------------------!
! Xiaoxu Guan at LSU HPC.                                        !
! March 22, 2017                                                 !
!----------------------------------------------------------------!

      program loops_out
      implicit none
      integer            :: i,j,nosize
      integer, parameter :: idp = 8, nsize = 100
      real(kind=idp)     :: tsum,time_start,time_end,omp_get_wtime
      real(kind=idp), dimension(0:nsize-1,0:nsize-1) :: a,b
      integer  :: omp_get_thread_num


      nosize = nsize
! initialize arrays: a, b, and c:
! column: i and row: j.
      do i = 0, nosize-1
      do j = 0, nosize-1
         a(j,i) = SIN( dble(j-i) )
         b(j,i) = COS( dble(i+j) )
      end do
      end do


      time_start = omp_get_wtime()

!$omp parallel do default(none) private(i,j,tsum) shared(nosize,a,b)
      loop_i: do i = 0, nosize-1
      loop_j: do j = 0, nosize-1
         tsum = a(j,i) + b(j,i)
         if( tsum > 0.5_idp) then 
             write(*,'(a,i2,a,i2,a,i2,a,f9.6)') &
               'My ID is ',omp_get_thread_num(), '  i = ', i,'  j = ', j, ' tsum =',tsum
             EXIT loop_j
         end if
      end do loop_j
      end do loop_i
!$omp end parallel do

      time_end = omp_get_wtime()
      write(*,'(a,f10.5)') 'elapsed time (seconds) = ', time_end - time_start


      end program loops_out

