!-------------------------------------------------------!
! The current OpenMP code is to:                        !
! (1) see how to parallalize nested loops.              !
! (2) use a naive version of matrix-matrix product.     !
!-------------------------------------------------------!
! Xiaoxu Guan at LSU HPC.                               !
! March 14, 2017                                        !
!-------------------------------------------------------!

      program nested_loops_v0
      implicit none
      integer            :: i,j,k,nosize
      integer, parameter :: idp = 8, nsize = 4000
      real(kind=idp)     :: temp,time_start,time_end,omp_get_wtime
      real(kind=idp), dimension(0:nsize-1,0:nsize-1) :: a,b,c


      nosize = nsize
! initialize arrays: a, b, and c:
! column: i and row: j.
      do i = 0, nosize-1
      do j = 0, nosize-1
         a(j,i) = SIN( dble(j-i) )
         b(j,i) = COS( dble(i+j) )
      end do
      end do

      c = 0.0_idp

! c = a x b 

      time_start = omp_get_wtime()

! only parallelize the outer loop with ijk:
!$omp parallel do default(none) private(i,j,k,temp) shared(nosize,a,b,c)
      do i = 0, nosize-1
      do j = 0, nosize-1
            temp = 0.0_idp
         do k = 0, nosize-1
            temp = temp + a(j,k) * b(k,i)
         end do
            c(j,i) = temp
      end do
      end do
!$omp end parallel do

      time_end = omp_get_wtime()

      write(*,'(a,f15.5)') 'c(0,0) = ', c(0,0)
      write(*,'(a,f10.5)') 'elapsed time (seconds) = ', time_end - time_start

!----------------------------------------------------------------------------!

      c = 0.0_idp

      time_start = omp_get_wtime()
! parallelize two outer loops with the same algorithm (ijk).
!$omp parallel do default(none) private(i,j,k,temp) shared(nosize,a,b,c)  &
!$omp collapse(2)
      do i = 0, nosize-1
      do j = 0, nosize-1
            temp = 0.0_idp
         do k = 0, nosize-1
            temp = temp + a(j,k) * b(k,i)
         end do
            c(j,i) = temp
      end do
      end do
!$omp end parallel do

      time_end = omp_get_wtime()

      write(*,'(a,f15.5)') 'c(0,0) = ', c(0,0)
      write(*,'(a,f10.5)') 'elapsed time (seconds) = ', time_end - time_start
      write(*,*)



      end program nested_loops_v0

