!------------------------------------------------------------!
! The current OpenMP code is to:                             !
! (1) see how to parallalize nested loops.                   !
! (2) see which loop should or should not be parallelized.   !
! (3) use a naive version of matrix-matrix product.          !
!------------------------------------------------------------!
! Xiaoxu Guan at LSU HPC.                                    !
! March 21, 2017                                             !
!------------------------------------------------------------!

      program nested_loops_v1
      implicit none
      integer            :: i,j,k,l,nosize
      integer, parameter :: idp = 8, nsize = 4000
      real(kind=idp)     :: tsum,time_start,time_end,omp_get_wtime
      real(kind=idp), dimension(0:nsize-1,0:nsize-1) :: a,b,c


      nosize = nsize
! initialize arrays: a, b, and c:
! column: i and row: j.
      do i = 0, nosize-1
      do j = 0, nosize-1
         a(j,i) = SIN( dble(j-i) )
         b(j,i) = COS( dble(i+j) )
      end do
      end do

      c = 0.0_idp

! c = a x b 

      time_start = omp_get_wtime()

      do i = 0, nosize-1
      do j = 0, nosize-1
            tsum = 0.0_idp
!$omp parallel do default(none) private(i,j,k) shared(nosize,a,b,c) reduction(+:tsum)
         do k = 0, nosize-1
            tsum = tsum + a(j,k) * b(k,i)
         end do
!$omp end parallel do
           c(j,i) = tsum
      end do
      end do

      time_end = omp_get_wtime()

      write(*,'(a,f15.5)') 'c(0,0) = ', c(0,0)
      write(*,'(a,f10.5)') 'elapsed time (seconds) = ', time_end - time_start


      end program nested_loops_v1

