!-------------------------------------------------------!
! The current OpenMP code is to:                        !
! (1) see how to parallalize nested loops.              !
! (2) understnad algorithm's importance.                !
! (3) use an optimized version matrix-matrix product.   !
!-------------------------------------------------------!
! Xiaoxu Guan at LSU HPC.                               !
! March 25, 2017                                        !
!-------------------------------------------------------!

      program nested_loops_v2
      implicit none
      integer            :: i,j,k,nosize
      integer, parameter :: idp = 8, nsize = 4000
      real(kind=idp)     :: temp,time_start,time_end,omp_get_wtime
!!      real(kind=idp), dimension(0:nsize-1,0:nsize-1) :: a,b,c
      real(kind=idp), dimension(:,:), allocatable :: a,b,c


      nosize = nsize
      ALLOCATE(a(0:nsize-1,0:nsize-1) )
      ALLOCATE(b(0:nsize-1,0:nsize-1) )
      ALLOCATE(c(0:nsize-1,0:nsize-1) )

! initialize arrays: a, b, and c:
! column: i and row: j.
      do i = 0, nosize-1
      do j = 0, nosize-1
         a(j,i) = SIN( dble(j-i) )
         b(j,i) = COS( dble(i+j) )
      end do
      end do

      c = 0.0_idp


! c = a x b 

      time_start = omp_get_wtime()

! stride-1 access for c and a matrices, while stride-0 for b matrix.
! only parallelize the outer loop in jki:
!$omp parallel do default(none) private(i,j,k) shared(nosize,a,b,c)
      do j = 0, nosize-1
      do k = 0, nosize-1
         do i = 0, nosize-1
            c(i,j) = c(i,j) + a(i,k) * b(k,j)
         end do
      end do
      end do
!$omp end parallel do

      time_end = omp_get_wtime()

      write(*,'(a,f15.5)') 'c(0,0) = ', c(0,0)
      write(*,'(a,f10.5)') 'elapsed time (seconds) = ', time_end - time_start
      write(*,*)


      end program nested_loops_v2

