! ======================================================!
! The current OpenMP code is to:                        !
! (1) do matrix-matrix products.                        !
! (2) see the behavior of the threaded MKL routine .    !
! (3) thread control at different levels.               !
! (4) precision: real DP.                               !
!-------------------------------------------------------!
! Xiaoxu Guan at LSU HPC.                               !
! May 12, 2017                                          !
!=======================================================!

      program dgemm_mkl
      implicit none

      integer, parameter :: nsize = 5000
      integer, parameter :: nsize2 = nsize*nsize
      integer, parameter :: iteration = 1600
      real(kind=8), parameter :: invs_nsize = 1.d0/dble(nsize)
      real(kind=8), parameter :: alpha = 1.d0, beta = 0.d0
      real(kind=8), dimension(:), allocatable ::  matrix_a, matrix_b, matrix_c
      character(len=1), parameter :: trans_a = 'N', trans_b = 'N'
      integer :: i,j,k,ii
      integer :: omp_get_num_threads

! allocate the matrices, same as c code.
      ALLOCATE( matrix_a(0:nsize2-1) )    
      ALLOCATE( matrix_b(0:nsize2-1) )    
      ALLOCATE( matrix_c(0:nsize2-1) )    

! initialize the matrices.
         k = -1
      do i = 0, nsize-1  ! column
      do j = 0, nsize-1  ! row
         k = k + 1
         ii = j * nsize + i
         matrix_a(k) = sin(dble(ii))*invs_nsize
         matrix_b(k) = cos(dble(ii))*invs_nsize
         matrix_c(k) = 0.d0 
      end do 
      end do 


!! repeat "iteration" times: C = A*B.
!$omp parallel do
      do k = 0, iteration-1
         write(*,'(1x,a,i3)') "Number of the threads = ", omp_get_num_threads()
         call DGEMM(trans_a,trans_b,nsize,nsize,nsize,alpha,matrix_a,nsize,   &
     &        matrix_b,nsize,beta,matrix_c,nsize)
      end do

! print a corner in the 1st row of the C matrix.
      do i = 0, 5
         write(*,'(es12.5,1x)')  matrix_c(i*nsize)
      end do

      write(*,*)

      DEALLOCATE(matrix_a,matrix_b,matrix_c)

      end program dgemm_mkl
